import * as vscode from 'vscode';
import { SERVICES_PATHS_PATH } from './core/config';
import MetadataProvider from './core/metadataProvider';
import { error } from './recorder/utils';
import { OPEN_BRIEFING_COMMAND } from './theia/command';
import { DepNodeProvider } from './theia/deadlockPanel';
import UserConfigTheia from './theia/userConfigTheia';
import BriefingView from './view/briefingView';
import Controller from './core/controller';
export function initViews() {
	new BriefingView();
}

export const userConfig = new UserConfigTheia();

export async function activate(context: vscode.ExtensionContext) {
	vscode.window.showInformationMessage('Bienvenue sur Deadlock!');

	vscode.window.registerUriHandler({
		handleUri(uri: vscode.Uri) {
			const queryParams: URLSearchParams = new URLSearchParams(uri.query);
			const action: string | null = queryParams.get('action');

			switch (action) {
				case 'open-challenge':
					Controller.launchMission(queryParams.get('missionId'));
					break;

				default:
					vscode.window.showInformationMessage('Aucune action trouvée!');
			}
		},
	});

	try {
		await userConfig.init();
	} catch (e) {
		error('Cannot init userConfig');
	}

	initViews();

	await vscode.commands.executeCommand(OPEN_BRIEFING_COMMAND.cmd);

	// @ts-ignore
	const deadlockPanelProvider = new DepNodeProvider(vscode.workspace.rootPath);
	vscode.window.registerTreeDataProvider('deadlockPanel', deadlockPanelProvider);

	MetadataProvider.loadPathsToEnvVariables();
	MetadataProvider.loadPathsToJson(`${SERVICES_PATHS_PATH}`);
}
