import GitMission from '../core/gitMission';
import { error, commitAndPushCode, CommitFrom, log } from './utils';
const async = require('async');
const fs = require('fs');

class Command {
	public still: boolean;

	constructor(public pid: number, private command: string) {
		this.pid = pid;
		this.command = command;

		this.still = true;
	}
}

export default class CommandRecorder {
	private commandsInProgress = new Map<number, Command>();
	private readonly queue: any;

	constructor(private gitMission: GitMission) {
		// create a queue object with concurrency 1
		this.queue = async.queue(async function (task) {
			await task.apply();
		}, 1);

		this.queue.error((err, _task) => {
			error('Error on task: ', err);
		});
	}

	somethingExecuted(pid, command) {
		if (!this.commandsInProgress.has(pid)) {
			this.commandsInProgress.set(pid, new Command(pid, command));
			try {
				this.queue.push(async () => await commitAndPushCode(this.gitMission, CommitFrom.Run));
			} catch (e) {
				console.error('Cannot send user code to git');
				console.error(e);
			}
		} else {
			this.commandsInProgress.get(pid)!.still = true;
		}
	}

	run() {
		let lastLineIndexWatched = 0;
		setInterval(() => {
			try {
				const trace = fs.readFileSync('/home/theia/.bash_history', 'utf8');
				const lines = trace.split(/\r?\n/);

				this.commandsInProgress.forEach((command) => (command.still = false));

				for (let index = lastLineIndexWatched; index < lines.length; index++) {
					const line = lines[index];
					if (
						line.indexOf('java') !== -1 ||
						line.indexOf('yarn serve') !== -1 ||
						line.indexOf('yarn start') !== -1 ||
						line.indexOf('npm run') !== -1 ||
						line.indexOf('npm start') !== -1
					) {
						this.somethingExecuted(index, line);
					}
				}
				lastLineIndexWatched = lines.length - 1;

				// clear command down (means still = false)
				this.commandsInProgress.forEach((command) => {
					if (!command.still) {
						this.commandsInProgress.delete(command.pid);
					}
				});
			} catch (e) {
				console.error(e);
			}
		}, 1300);
	}
}
