import CommandRecorder from './command-recorder';
import GitMission from '../core/gitMission';
import UserConfigNode from './userConfigNode';
import { PROJECT_SRC_PATH, PROJECT_THEIA_PATH } from '../core/config';
import { copyProjectSources, clearFilesExceptGit, log, error, renameTempToUserGitFiles } from './utils';
import UserConfig from '../core/userConfig';

export default class Recorder {
	async setupProject(userConfig: UserConfig, gitMission?: GitMission) {
		log('Setup user project..');

		if (!userConfig.isProfessor()) {
			await copyProjectSources(PROJECT_SRC_PATH, PROJECT_THEIA_PATH, ['.git/']);

			if (gitMission) {
				renameTempToUserGitFiles(PROJECT_THEIA_PATH, gitMission.author);

				log('Starting CommandRecorder..');
				new CommandRecorder(gitMission).run();
			} else {
				error('Cannot start command recorder, gitMission not found');
			}
		} else {
			await copyProjectSources(PROJECT_SRC_PATH, PROJECT_THEIA_PATH);
		}
	}

	async setupFromRemoteRepo(gitMission: GitMission) {
		log('Check if remote repo exist');
		const isRemoteRepoExist = await gitMission.isRemoteRepoExist();
		if (isRemoteRepoExist) {
			// rm all except git directory pull remote code and setup
			log('Cleaning files to pull repo');

			await clearFilesExceptGit(PROJECT_SRC_PATH);

			log('Pulling user repo');
			await gitMission.pull();
		}
	}

	async run() {
		const userConfig = new UserConfigNode();

		let gitMission;

		try {
			log('Init User config');
			await userConfig.init();
			log('Init GitMission');
			gitMission = await new GitMission(userConfig).init();
			await this.setupFromRemoteRepo(gitMission);
		} catch (e) {
			error('Cannot setup user repo.');
			error(e);
		}
		try {
			await this.setupProject(userConfig, gitMission);
		} catch (e) {
			error('Error while setup project sources');
			error(e);
		}
	}
}

new Recorder().run();
