import UserConfigNode from './userConfigNode';
import GitMission from '../core/gitMission';
import { PROJECT_SRC_PATH, PROJECT_THEIA_PATH } from '../core/config';

import { log, error, commitAndPushCode, CommitFrom } from './utils';
const util = require('util');
const exec = util.promisify(require('child_process').exec);

async function containsDiff() {
	try {
		// https://man7.org/linux/man-pages/man1/diff.1.html
		// Exit status is 0 if inputs are the same, 1 if different, 2 if trouble.
		await exec(`diff -qr ${PROJECT_SRC_PATH} ${PROJECT_THEIA_PATH}`);
		// When status code is 0 exec does not fail
	} catch (result) {
		// when status code is > 0
		if (result.code === 1) {
			const stdout = result.stdout;
			if (stdout.indexOf('Files ') !== -1 || stdout.indexOf('Only in /home/project/') !== -1) {
				// if user created new file or added a directory
				return true;
			}
		} else {
			// print error
			error(result.stderr);
		}
	}
	return false;
}

(async () => {
	try {
		log('Container will die');

		const userConfig = new UserConfigNode();
		await userConfig.init();

		if (!userConfig.isProfessor()) {
			log('Save user code..');
			const gitMission = await new GitMission(userConfig).init();
			if (await containsDiff()) {
				await commitAndPushCode(gitMission, CommitFrom.Auto);
			}
		}
	} catch (e) {
		error('Cannot push user code at the end..');
		error(e);
	}
})();
