import * as vscode from 'vscode';
import * as path from 'path';
import { TreeItemCollapsibleState } from 'vscode';
import { OPEN_BRIEFING_COMMAND } from './command';

const DOCUMENTATION_LABEL = 'Documentation';

export class DepNodeProvider implements vscode.TreeDataProvider<Action> {
	private _onDidChangeTreeData: vscode.EventEmitter<Action | undefined | void> = new vscode.EventEmitter<
		Action | undefined | void
	>();
	// @ts-ignore
	readonly onDidChangeTreeData: vscode.Event<Action | undefined | void> = this._onDidChangeTreeData.event;

	constructor(private workspaceRoot: string) {}

	refresh(): void {
		this._onDidChangeTreeData.fire();
	}

	private root(): Action[] {
		return [new Action(DOCUMENTATION_LABEL, TreeItemCollapsibleState.Collapsed, 'folder')];
	}

	getTreeItem(element: Action): vscode.TreeItem {
		return element;
	}

	getChildren(element?: Action): Thenable<Action[]> {
		if (!element) {
			return Promise.resolve(this.root());
		} else if (element.label === DOCUMENTATION_LABEL) {
			return Promise.resolve([
				new Action(
					'Briefing',
					TreeItemCollapsibleState.None,
					'document',
					OPEN_BRIEFING_COMMAND.asVsCodeCommand(),
				),
			]);
		}

		return Promise.resolve([]);
	}
}

export class Action extends vscode.TreeItem {
	constructor(
		public readonly label: string,
		public readonly collapsibleState: vscode.TreeItemCollapsibleState,
		public readonly iconName: string,
		public readonly command?: vscode.Command,
	) {
		super(label, collapsibleState);
	}

	get tooltip(): string {
		return this.label;
	}

	get description(): string {
		return '';
	}

	iconPath = {
		light: path.join(__filename, '..', '..', 'resources', 'light', `${this.iconName}.svg`),
		dark: path.join(__filename, '..', '..', 'resources', 'dark', `${this.iconName}.svg`),
	};

	contextValue = 'dependency';
}
